@@ -424,7 +424,22 @@ class MemoryAPI:
         self.Sql = SQLAPI(self)
         self.MEMORY = []
         self.LOCALMEMORY = []
+        self.USE_PERSISTANT = False
         self._lock = Lock()
+        self.path = ""
+        
+    def EnableMemoryConsistent(self, path: str, option: bool = True):
+        self.ConsistentLoad(path)
+            
+    def Update(self):
+        if self.USE_PERSISTANT and self.path:
+            self.MEMORY = []
+            self.MEMORY = self.ConsistentLoad(self.path)
+        
+    def Reload(self):
+        
+        if self.USE_PERSISTANT and self.path:
+            self.ConsistentLoad(self.path)
 
     def KnowThis(self, name: str, meta: dict):
         """Save Data in Memory
@@ -487,7 +502,7 @@ class MemoryAPI:
                 return True
         return False
     
-    def Tell(self, name: str, use_value: bool = False, keyword: str = None, _dict: str = None, dict_in_dict: bool = False):
+    def MakeGlobal(self, name: str, use_value: bool = False, keyword: str = None, _dict: str = None, dict_in_dict: bool = False):
         """Promote a Memory entry into the top-level App.__global__.
 
         If use_value is True and keyword/_dict provided, extract that value from the stored meta.
@@ -551,6 +566,14 @@ class MemoryAPI:
                     global_mem[key] = value
                 return True
         return False
+    
+    def ConsistentSave(self, name: str):
+        self.LocalMemory.WriteLocal(self.path, self.Remember(name))
+        
+    def ConsistentLoad(self, path):
+        self.path = path
+        data = self.LocalMemory.ReadLocal(path)
+        self.MEMORY = data
 
 class LocalMemoryAPI:
     """
@@ -632,59 +655,227 @@ class LocalMemoryAPI:
             return inserts
         
 class SQLAPI:
+    """
+    SQLAPI - Eine moderne, kontrollierte SQL-API mit Framework-Feeling.
+    Designed für Toolos 3.0+ Framework Integration.
+
+    > Features:
+      - Datenbankprüfung & Erstellung
+      - Tabellenmanagement (Existenz, Create, Drop)
+      - CRUD + Search Funktionen
+      - Automatisches Path-Handling über Settings
+      - Einfache Syntax: self.Sql.TableExists("users")
+    """
+
     def __init__(self, app):
-        import sqlite3
+        import sqlite3, os
         self.sqlite3 = sqlite3
         self.App = app
+        # Settings-Attribut korrekt auflösen
+        if hasattr(app, "Settings"):
+            self.db_path = getattr(app.Settings, "SETTINGSPATH", "settings.db")
+        elif hasattr(app, "App") and hasattr(app.App, "Settings"):
+            self.db_path = getattr(app.App.Settings, "SETTINGSPATH", "settings.db")
+        else:
+            self.db_path = "settings.db"
+        self.connection = None
+        self.cursor = None
+    # ? #############################################
+    # ? BASIC CONNECTION
+    # ? #############################################
 
-    def Create(self, table: str, data: dict):
+    def Connect(self, path: str = None):
+        """Öffnet eine Connection (automatisch wenn nicht offen)."""
+        if not path:
+            path = self.db_path
         try:
-            conn = self.sqlite3.connect(self.App.Settings.SETTINGSPATH)
-            cursor = conn.cursor()
+            self.connection = self.sqlite3.connect(path, check_same_thread=False)
+            self.cursor = self.connection.cursor()
+            self.db_path = path
+            return True
+        except Exception as e:
+            print(f"[SQL] Connection error: {e}")
+            return False
+
+    def Close(self):
+        """Schließt aktuelle Connection"""
+        if self.connection:
+            self.connection.close()
+            self.connection = None
+            self.cursor = None
+            return True
+        return False
+
+    # ? #############################################
+    # ? DATABASE CONTROL
+    # ? #############################################
+
+    def DbExists(self, path: str = None) -> bool:
+        """Check ob DB-Datei existiert"""
+        import os
+        return os.path.exists(path or self.db_path)
+
+    def DbSearch(self, directory: str = ".") -> list:
+        """Sucht nach SQLite-Dateien im Ordner"""
+        import os
+        return [f for f in os.listdir(directory) if f.endswith(".db") or f.endswith(".sqlite")]
+
+    def DeleteDb(self, path: str = None) -> bool:
+        """Löscht eine Datenbank"""
+        import os
+        path = path or self.db_path
+        if os.path.exists(path):
+            os.remove(path)
+            return True
+        return False
+
+    # ? #############################################
+    # ? TABLE MANAGEMENT
+    # ? #############################################
+
+    def TableExists(self, name: str) -> bool:
+        """Prüft, ob Tabelle existiert"""
+        try:
+            self.Connect()
+            self.cursor.execute(
+                "SELECT name FROM sqlite_master WHERE type='table' AND name=?;", (name,)
+            )
+            return self.cursor.fetchone() is not None
+        except Exception as e:
+            print(f"[SQL] TableExists Error: {e}")
+            return False
+
+    def TableCreate(self, name: str, columns: dict):
+        """Erstellt neue Tabelle: { 'id': 'INTEGER PRIMARY KEY', 'name': 'TEXT', ... }"""
+        try:
+            self.Connect()
+            col_str = ', '.join([f"{col} {typ}" for col, typ in columns.items()])
+            self.cursor.execute(f"CREATE TABLE IF NOT EXISTS {name} ({col_str});")
+            self.connection.commit()
+            print(f"[SQL] Table '{name}' created or already exists.")
+            return True
+        except Exception as e:
+            print(f"[SQL] TableCreate Error: {e}")
+            return False
+
+    def TableDelete(self, name: str):
+        """Löscht Tabelle"""
+        try:
+            self.Connect()
+            self.cursor.execute(f"DROP TABLE IF EXISTS {name};")
+            self.connection.commit()
+            return True
+        except Exception as e:
+            print(f"[SQL] TableDelete Error: {e}")
+            return False
+
+    def TableList(self) -> list:
+        """Listet alle Tabellen in der DB"""
+        try:
+            self.Connect()
+            self.cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
+            return [t[0] for t in self.cursor.fetchall()]
+        except Exception:
+            return []
+
+    # ? #############################################
+    # ? CRUD (Create, Read, Update, Delete)
+    # ? #############################################
+
+    def Insert(self, table: str, data: dict):
+        """Fügt Datensatz hinzu"""
+        try:
+            self.Connect()
             columns = ', '.join(data.keys())
-            placeholders = ', '.join('?' * len(data))
+            placeholders = ', '.join(['?'] * len(data))
             sql = f"INSERT INTO {table} ({columns}) VALUES ({placeholders})"
-            cursor.execute(sql, tuple(data.values()))
-            conn.commit()
-            return cursor.lastrowid
+            self.cursor.execute(sql, tuple(data.values()))
+            self.connection.commit()
+            return self.cursor.lastrowid
         except Exception as e:
+            print(f"[SQL] Insert Error: {e}")
             return None
 
-    def Read(self, table: str, record_id: int):
-        """Retrieve a record from the specified table by ID."""
+    def Read(self, table: str, where: dict = None):
+        """Liest Daten (optional mit where dict)"""
         try:
-            conn = self.sqlite3.connect(self.App.Settings.SETTINGSPATH)
-            cursor = conn.cursor()
-            sql = f"SELECT * FROM {table} WHERE id = ?"
-            cursor.execute(sql, (record_id,))
-            return cursor.fetchone()
+            self.Connect()
+            if where:
+                conditions = ' AND '.join(f"{k}=?" for k in where.keys())
+                sql = f"SELECT * FROM {table} WHERE {conditions}"
+                self.cursor.execute(sql, tuple(where.values()))
+            else:
+                sql = f"SELECT * FROM {table}"
+                self.cursor.execute(sql)
+            return self.cursor.fetchall()
         except Exception as e:
-            return None
+            print(f"[SQL] Read Error: {e}")
+            return []
 
-    def Update(self, table: str, record_id: int, data: dict):
-        """Update an existing record in the specified table."""
+    def Update(self, table: str, updates: dict, where: dict):
+        """Updatet Datensätze"""
         try:
-            conn = self.sqlite3.connect(self.App.Settings.SETTINGSPATH)
-            cursor = conn.cursor()
-            updates = ', '.join(f"{key} = ?" for key in data.keys())
-            sql = f"UPDATE {table} SET {updates} WHERE id = ?"
-            cursor.execute(sql, (*data.values(), record_id))
-            conn.commit()
-            return cursor.rowcount
+            self.Connect()
+            update_str = ', '.join(f"{k}=?" for k in updates.keys())
+            where_str = ' AND '.join(f"{k}=?" for k in where.keys())
+            sql = f"UPDATE {table} SET {update_str} WHERE {where_str}"
+            self.cursor.execute(sql, tuple(updates.values()) + tuple(where.values()))
+            self.connection.commit()
+            return self.cursor.rowcount
         except Exception as e:
-            return None
+            print(f"[SQL] Update Error: {e}")
+            return 0
 
-    def Delete(self, table: str, record_id: int):
-        """Delete a record from the specified table."""
+    def Delete(self, table: str, where: dict = None):
+        """Löscht Datensätze (optional mit WHERE)"""
         try:
-            conn = self.sqlite3.connect(self.App.Settings.SETTINGSPATH)
-            cursor = conn.cursor()
-            sql = f"DELETE FROM {table} WHERE id = ?"
-            cursor.execute(sql, (record_id,))
-            conn.commit()
-            return cursor.rowcount
+            self.Connect()
+            if where:
+                conditions = ' AND '.join(f"{k}=?" for k in where.keys())
+                sql = f"DELETE FROM {table} WHERE {conditions}"
+                self.cursor.execute(sql, tuple(where.values()))
+            else:
+                sql = f"DELETE FROM {table}"
+                self.cursor.execute(sql)
+            self.connection.commit()
+            return self.cursor.rowcount
         except Exception as e:
-            return None
+            print(f"[SQL] Delete Error: {e}")
+            return 0
+
+    # ? #############################################
+    # ? SEARCH & META
+    # ? #############################################
+
+    def TableSearch(self, table: str, column: str, keyword: str):
+        """Einfacher Textsuche in Spalte"""
+        try:
+            self.Connect()
+            sql = f"SELECT * FROM {table} WHERE {column} LIKE ?"
+            self.cursor.execute(sql, (f"%{keyword}%",))
+            return self.cursor.fetchall()
+        except Exception as e:
+            print(f"[SQL] TableSearch Error: {e}")
+            return []
+
+    def FetchColumns(self, table: str):
+        """Gibt alle Spaltennamen der Tabelle zurück"""
+        try:
+            self.Connect()
+            self.cursor.execute(f"PRAGMA table_info({table});")
+            return [col[1] for col in self.cursor.fetchall()]
+        except Exception as e:
+            print(f"[SQL] FetchColumns Error: {e}")
+            return []
+
+    def Count(self, table: str) -> int:
+        """Zählt Einträge"""
+        try:
+            self.Connect()
+            self.cursor.execute(f"SELECT COUNT(*) FROM {table}")
+            return self.cursor.fetchone()[0]
+        except Exception:
+            return 0
 
 class ImageAPI:
     def __init__(self, app):
@@ -1097,16 +1288,46 @@ from dataclasses import dataclass
 from threading import Lock
 
 class StateType(Enum):
-    STEP_1 = "step_1"
-    STEP_2 = "step_2"
-    STEP_3 = "step_3"
-    STEP_4 = "step_4"
-    STEP_5 = "step_5"
+    STEP1 = "step_1"
+    STEP2 = "step_2"
+    STEP3 = "step_3"
+    STEP4 = "step_4"
+    STEP5 = "step_5"
     EXIT = "exit"
     MAIN_MENU = "main_menu"
     FIRST_ENTRY = "first_entry"
     LOGIN = "login"
     VERIFIED = "verified"
+    START = "start"
+    CHECK = "check"
+    MAIN = "main"
+    RED = "red"
+    GREEN = "green"
+    NEW = "new"
+    USER = "user"
+    REGISTER = "register"
+    # Buttons
+    DKEY = "d_key"
+    WKEY = "w_key"
+    AKEY = "a_key"
+    SKEY = "s_key"
+    QKEY = "q_key"
+    EKEY = "e_key"
+    LEFT = "arrow_left"
+    RIGHT = "arrow_right"
+    DOWN = "arrow_down"
+    UP = "arrow_up"
+    #Game States
+    KILLED = "killed"
+    ENTITY = "entity"
+    PLAYER = "player"
+    MOVING = "moving"
+    JUMPING = "jumping"
+    ATTACKING = "attacking"
+    LOW_HEALTH = "low_health"
+    
+    
+    
 
 @dataclass
 class State:
@@ -2353,6 +2574,12 @@ class Api:
                 self.Memory.KnowThis(name, self.Collect(id))
                 self.Delete(id)
                 
+    def Update(self, no_setting: bool=False, no_memory: bool=False):
+        """Updates All API's
+        > This means calling all update() func """
+        if not no_setting: self.Settings.Update()
+        if not no_memory: self.Memory.Update()
+                
     def Quit(self, enable_animation: bool=True, sleep_time_per_step: list=[1, 1, 0.4], steps: list=['shutdown engines...', 'shutdown Memory...', 'Finishing Work']):
 
         if enable_animation:
@@ -2545,16 +2772,16 @@ class Driver(Api):
         self.Server = ServerDriver(self)
         
 
-# def test():
-#     app = Api(settings={
-#         "language": "de",
-#         "CACHEPATH": "./cache",
-#         "TEMPPATH": "./temp",
-#         "PACKAGEPATH": "./packages",
-#         "LOGPATH": "./logs"
-#         })
-#     if app.StateMachine.sIsState(app.StateMachine.StateType.FIRST_ENTRY.value):
-#         print("First Entry State")
+def test():
+    app = Api(settings={
+        "language": "de",
+        "CACHEPATH": "./cache",
+        "TEMPPATH": "./temp",
+        "PACKAGEPATH": "./packages",
+        "LOGPATH": "./logs"
+        })
+    if app.StateMachine.sIsState(app.StateMachine.StateType.FIRST_ENTRY.value):
+        print("First Entry State")
         
-# if __name__ == "__main__":
-#     test()
\ No newline at end of file
+if __name__ == "__main__":
+    test()
\ No newline at end of file
